@@ -13,6 +13,7 @@ use cargo::{
 use retry::{delay, retry_with_index};
 use semver::Version;
 use serde::{Deserialize, Serialize};
+use toml_edit::Document;
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct ArmoryTOML {
@@ -34,11 +35,53 @@ pub fn save_armory_toml(workspace_dir: &Path, armory_toml: &ArmoryTOML) {
         .unwrap();
 }
 
+#[derive(Debug, Clone, Serialize, Deserialize)]
+struct WorkspaceManifest {
+    pub workspace: WorkspaceDefinition,
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+struct WorkspaceDefinition {
+    pub members: Vec<String>,
+}
+
+fn update_member_deps(dir: &Path, version: &Version) {
+    let workspace_toml: WorkspaceManifest = toml::from_slice(
+        &fs::read_to_string(dir.join("Cargo.toml"))
+            .unwrap()
+            .as_bytes(),
+    ).unwrap();
+
+    for member in workspace_toml.workspace.members {
+        let member_dir = dir.join(&member);
+        let member_toml = fs::read_to_string(member_dir.join("Cargo.toml")).unwrap();
+        let mut member_toml = member_toml.parse::<Document>().unwrap();
+
+        member_toml["package"]["version"] = toml_edit::value(version.to_string());
+        let deps = member_toml.get_mut("dependencies").map(|deps| deps.as_table_mut());
+        match deps {
+            Some(Some(table)) => {
+                for (name, value) in table.iter_mut() {
+                    if let Some(dep) = value.as_table_mut() {
+                        if let Some(Some(_)) = dep.get("path").map(|dep| dep.as_str()) {
+                            // this is a local dependency, so we will need to update the version
+                            dep["version"] = toml_edit::value(version.to_string());
+                            dbg!(name);
+                        }
+                    }
+                }
+            }
+            _ => {}
+        }
+    }
+}
+
 pub fn publish_workspace(dir: &Path, version: &Version) {
     let mut cfg = Config::default().unwrap();
     cfg.set_values(cfg.load_values().unwrap()).unwrap();
     cfg.load_credentials().unwrap();
 
+    update_member_deps(dir, version);
     // let dir = Path::new("/Users/albert/repos/armory/");
     let workspace_toml = dir.clone().join("Cargo.toml");
     let mut workspace = Workspace::new(&workspace_toml, &cfg).unwrap();
@@ -124,9 +167,8 @@ fn publish_crate(
             publish_crate(workspace, cfg, local_dep, all_packages, already_published);
         }
     }
-    println!("ARMORY: publishing {}", current_package);
 
-    retry_with_index(delay::Fibonacci::from_millis(2000), |current_try| {
+    retry_with_index(delay::Fibonacci::from_millis(2500), |current_try| {
         match cargo::ops::publish(
             workspace,
             &PublishOpts {
@@ -145,9 +187,12 @@ fn publish_crate(
         ) {
             Ok(_) => Ok(()),
             Err(e) => {
-                if current_try > 6 {
+                if current_try > 4 {
                     panic!("ARMORY: failed to publish {} after {} attempts: {:#?}",
                             current_package, current_try, e);
+                } else {
+                    println!("ARMORY: failed to publish {} after {} attempts: {:#?}",
+                        current_package, current_try, e);
                 }
                 Err(e)
             }
